<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class Admin_AdController extends Controller
{
    /**
    * set access privileges
    */
    function init()
    {        
        $this->acl->allow('administrator', $this->name, "*");
         
        if(!$this->acl->isAllowed($this->session->get("role"), $this->name, $this->action))
        {
            $this->redirect(FrontController::getRewrittedUrl("/admin/main/logIn"));  
        }
    }   
      
    /**
    * Display all criterions - managment page  
    */
    function indexAction()
    {
        $this->set("adCriterias", $this->adCriteria->findAll());                 
    }
    
    /**
    * Get criterions Ids for specified site return it to ajax query 
    */
    function getAdsOnPageAction()
    {
        $ads = $this->ad->getAdsOnPage($this->request->page);        
        $this->set($ads);
        $this->viewClass = "JsonView";
    }
    
    /**
    * Display form to edit ads on specified site 
    */
    function inAction($page)
    {        
        //list of sites which can be edited
        $pages = array("news", "topHits", "topNotes", "topRank", "topReferrers", "search",
                       "predefineCategoriesAndSites", "predefineKeyword", "index", "keywordLetter");
        
        if(!in_array($page, $pages))
        {
            $this->return404();
        }
        
        $itemsPerPage = 0;
        
        //determine siteItems slots count to display on this ads page
        switch($page)
        {
            case "news": $itemsPerPage = 3; break;
            case "topHits": $itemsPerPage = Config::get("maxTopHitsCount"); break; 
            case "topNotes": $itemsPerPage = Config::get("maxTopNotesCount"); break; 
            case "topRank": $itemsPerPage = Config::get("maxTopRankCount"); break; 
            case "topReferrers": $itemsPerPage = Config::get("maxTopReferrersCount"); break; 
            case "search": $itemsPerPage = Config::get("sitesPerPageInSearch"); break; 
        }
        
        $this->set("itemsPerPage", $itemsPerPage);
        
        //get all accessible criterions to choose
        $this->set("adCriterias", $this->adCriteria->getSelectList());  

        //set adPage name to js know what is edited
        $this->set("adPage", $page);
        $this->set("adPageDescription", str_replace("_", " ", NameTool::underscore($page)));
        
        //if non standard page load indyvidual files
        if(in_array($page, array("predefineCategoriesAndSites", "predefineKeyword", "index", "keywordLetter")))
        {
            if($page == "predefineCategoriesAndSites")
            {
                $this->set("adPage", "predefineCategory");
                $this->set("adPage2", "predefineSite");
            }
            
            $this->viewFile = "pages/".$page;
        }
    }
    
    /**
    * Delete specified ad criterion
    */
    function deleteCriteriaAction($adCriterionId)
    {
        $this->adCriteria->delByPk($adCriterionId);
        
        //delete cache ads cache informations to be fresh
        Cacher::getInstance()->clean("tag", array("adCriteria"));
        $this->redirect($this->moduleLink());
    }
    
    /**
    * Display ad criterion edit form
    */
    function editCriteriaAction($adCriterionId)
    {
        $this->set("adCriteria", $this->adCriteria->findByPk($adCriterionId));
    }
    
    /**
    * saveEdited criterion
    */
    function saveAdCriteriaEditAction($adCriterionId)
    {
        //update criterai in db
        $data = $this->request->getArray(array("name", "htmlContent"));
        $this->adCriteria->updateByPk($data, $adCriterionId);
        
        //delete cache ads cache informations to be fresh
        Cacher::getInstance()->clean("tag", array("adCriteria"));    
        $this->redirect($this->moduleLink());
    }
    
    /**
    * set ad for specified place on page
    */
    function updateAction()
    {
        $data = $this->request->getArray(array("place", "page", "adCriterionId"));
        
        $this->ad->setAdOnPage($data);
        
        //if predefinition was selected so force general switch
        if($data['place'] == "predefine")
        {
            $data['place'] = "general";
            $this->ad->setAdOnPage($data);
        }

        //delete cache ads cache informations to be fresh
        Cacher::getInstance()->clean("tag", array("ad"));   
        
        $this->set("result", true);
        $this->set("message", _t("Saved"));
        $this->viewClass = "JsonView";
    }
    
    /**
    * Save new criterion
    */
    function saveNewCriteriaAction()
    {
        $adcriteria = new AdCriteriaRecord();
        $adcriteria->name = $this->request->name;
        $adcriteria->htmlContent = $this->request->htmlContent;
        $adcriteria->save();  
        
        $this->redirect($this->moduleLink());                                 
    }
} 

?>