<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

/**
 * Class which is responsible for url routing, executing controllers actions
 */
class FrontController extends Controller
{
    private static $routes = array();
    private static $instance = null;
    private static $urlReplacements = array();
    private static $urlPatterns = array();
    private static $executeDir = "";
    private static $websiteDir;
    private static $debug = false;
    private static $frontRewritesPatterns = array();
    private static $frontRewritesReplacements = array();
    
    /**
     * Returns an instance of FrontController object
     * @return FrontController 
     */
    public static function getInstance()
    {
        if(self::$instance === null)
        {
            self::$instance = new self();
        }

        return self::$instance;
    }
    
    /**
     * Generates the standard FrontController object
     */   
    public function __construct()
    {
        parent::__construct();

        $this->localDir = "";
        $this->loadSettings();
          
        self::$websiteDir = substr(strstr(preg_replace("#^http://#", "", Config::get("siteRootUrl")), "/"), 1);
        if(self::$websiteDir)self::$websiteDir = "/".rtrim(self::$websiteDir, "/");
        
        if(Config::get("urlRewriting"))
        {      
            require(CODE_ROOT_DIR."config/rewrite.php");
            self::$urlPatterns = array_keys($rewrites);
            self::$urlReplacements = array_values($rewrites);
            self::$frontRewritesPatterns = array_keys($frontRewrites);
            self::$frontRewritesReplacements = array_values($frontRewrites);
        }
        
        $dir = new DirectoryIterator(CODE_ROOT_DIR."plugins/");
        $routes = array();
        
        foreach($dir as $file)
        {
            if($file->isDot())continue;
            $executeDir = 'plugins/'.$file->getFileName();
            $routes[$executeDir.'*'] = array("executeDir" => $executeDir);
        }
        
        
         $routes = $routes + array("admin.*"      => array("executeDir" => "admin"),
                                   "moderation.*" => array("executeDir" => "moderation"),
                                   ".*"           => array("executeDir" => ""));
         
         self::$routes = $routes;

    }
    
    /**
     * Retrieve main settings for application
     * @return array
     */ 
    public function loadSettings()
    {
        $cache = Cacher::getInstance();

        if(($settings = $cache->load("settings")) === null)
        {
            $settings = $this->setting->findAll(new Criteria(), "`key`, `value`");
            $cache->save($settings, null, null, array("setting"));
        }
        
        foreach($settings as $row)
        {
            Config::set($row['key'], $row['value']);
        }

    }
    
    public static function getHostNameFromUrl($url)
    {
        $regs = array();
        
        $hostname = '';
        
        if (ereg('^http://([^/]*)$', $url))
            $hostname = $url;
        else
        {
            ereg('^http://([^/]*)/(.*)$', $url, $regs);
            $hostname = 'http://' . $regs[1];
        }
            
        return $hostname;
    }
    
    public static function changeFr($title)
    {
        $specialChars = array( ' ','\'','Ŕ','Á','Â','Ă','Ä','Ĺ','Ć','ŕ','à','á','â','ă','ä','ĺ','ć','Č','É','Ę','Ë','č','è','é','ę','ë','Ě','Í','Î','Ď','ě','í','î','ď','Ň','Ó','Ô','Ő','Ö','Ř','ň','ó','ô','ő','ö','ř','Ů','Ú','Ű','Ü','ů','ú','ű','ü','ß','Ç','ç','Đ','đ','Ń','ń','Ţ','ţ','Ý' );
        $normalChars  = array( '-', '-','A','A','A','A','A','A','A','a','a','a','a','a','a','a','a','E','E','E','E','e','e','e','e','e','I','I','I','I','i','i','i','i','O','O','O','O','O','O','o','o','o','o','o','o','U','U','U','U','u','u','u','u','B','C','c','D','d','N','n','P','p','Y' );

        $stringMaxLength = 70;
        
        $newTitle = 
            substr( 
                ereg_replace( 
                    "\-$", 
                    '', 
                    ereg_replace( 
                        "\-+", 
                        '-', 
                        strtolower( 
                            ereg_replace( 
                                    "[^A-Za-z0-9]", 
                                    '-', 
                                           
                                    str_replace(
                                        $specialChars, 
                                        $normalChars, 
                                        htmlspecialchars_decode($title)
                                        )
                                    ) 
                            ) 
                        ) 
                    ), 
                    0, 
                    $stringMaxLength
                );  

        return $newTitle;
    }
    
    public static function charsCallback($matches)
    {
        $newUrl = $matches[1].self::changeFr($matches[2]);
        if(isset($matches[3]))$newUrl .= $matches[3];
        return $newUrl; 
    }
    
    public static function getRewrittedUrl($url, $absolute = true)
    {
        $changes = array("#^(/site/details/.*/)(.*)$#",
                         "#^(/site/category/.*/)(.*)(/.*)$#",
                         "#^(/site/keyword/.*/)(.*)(/.*)$#"); 
         
        $url = preg_replace_callback($changes, array('FrontController', "charsCallback"), $url);

        if(Config::get("urlRewriting"))
        {
            $url = preg_replace(self::$urlPatterns, self::$urlReplacements, $url);
        }
        else
        {
            $url = "/index.php".$url; 
        }
        
        $url = ($absolute) ? self::$websiteDir.$url : ltrim($url, '/'); 
        return $url;  
    }
    
    public static function getResourceUrl($url, $absolute = true)
    {        
        $url = ($absolute) ? self::$websiteDir.$url : ltrim($url, '/'); 
        return $url;  
    }

    /**
     * Extract route part from request url
     * @return string
     */     
    private function getUrlParams()
    {                    
        $removePattern = '#^'.preg_quote(self::$websiteDir).'(/index.php)?#';
        $url = preg_replace($removePattern, '', $_SERVER["REQUEST_URI"]);
  
        $url = preg_replace(self::$frontRewritesPatterns, self::$frontRewritesReplacements, $url); 

        return $url;
    }
    
    public function setExecuteDir($dir)
    {
        self::$executeDir = $dir;
    }
    
    /**
     * Throws error if something is missed
     */         
    protected function throwError($msg)
    {    
        if(self::$debug)
        {            
            $msg = str_replace(CODE_ROOT_DIR, "", $msg);
            $this->set("error", $msg);
            $this->viewFile = "error";
            echo $this->render();
        }
        else
        {
            $this->return404();
        }
    }
      
    public function addRoute($routePattern, $routeOptions)
    {
        self::$routes[$routePattern] = $routeOptions;
    }

    /**
     * Return controller dir and routing path for specified url
     *
     * @param string $url
     * @return array
     */ 
    public function getRoute($url)
    {
        foreach(self::$routes as $routePattern => $routeOptions)
        {
            if(preg_match('#'.$routePattern.'#', $url))
            {
                return $routeOptions;
            }
        } 
    }
    
    /**
     * Dispatch an HTTP request to a controller/action.
     *
     * @param string $url Optional argument Url with routing informatins
     */
    public function dispatch($url = false, $return = false)
    {                 
        if(!$url)$url = $this->getUrlParams();
        $url = trim($url, "/");
        
        $route = $this->getRoute($url); 
        
        if($route['executeDir'])
        {
            $url = preg_replace("#^".$route['executeDir']."/?#", "", $url);
        }           
        
        $params = explode("/", $url);
        $index = 0;
        
        $route['path'] = $route['executeDir'] ? $route['executeDir']."/" : ""; 
        $this->setExecuteDir($route['executeDir']);
        $controllerDir = CODE_ROOT_DIR.$route['path']."controllers/";
        
        Config::set("LANGUAGES_PATH", $route['path']."languages/");

        $controllerName   = empty($params[$index]) ? "" : $params[$index];
        $index++;
        $controllerAction = empty($params[$index]) ? "" : $params[$index];
        
        if(empty($controllerName) && !empty($controllerAction))$this->return404();
        
        if(empty($controllerName))$controllerName = "Main";
        if(empty($controllerAction))$controllerAction = "index";
        
        $controller_action_method = $controllerAction."Action";
        $controllerClassName = NameTool::getControllerClassName($controllerName, $route['executeDir']); 
        
        $controllerFileName = NameTool::getControllerFileName($controllerName);
        $controller_path = $controllerDir.$controllerFileName;
        
        if(!isset($controllers[$controllerClassName]))
        {
            if(!file_exists($controller_path))
            {
                return $this->throwError("Missed controller file: $controller_path");
            } 
            
            require_once($controller_path);

            if(!class_exists($controllerClassName))
            {
                return $this->throwError("Missed controller class". ": $controllerClassName");
            }  
            
            static $controllers = array();
 
            $controllers[$controllerClassName] = new $controllerClassName($this);
        }
        
        $controller = $controllers[$controllerClassName]; 
        $controller->action = $controllerAction;
        $controller->localDir = $route['path'];
        
        if(!is_callable(array(&$controller, $controller_action_method)))
        {
            return $this->throwError("Missed action method". ": $controller_action_method");
        }  
        
        if(is_callable(array(&$controller, "init")))
        {
            call_user_func_array(array(&$controller, "init"), array());
        }
        
        $params = array_slice($params, 2);
            
        $result = call_user_func_array(array(&$controller, $controller_action_method), $params);

        if($return || $result !== null)
        {
            return ($result !== null) ? $result : $controller->render();
        }
        else
        {
            echo $controller->render();
        } 

    }
}
?>